from typing import Dict, IO, List, Tuple

import sc2reader
import techlabreactor


def serialise_chart_data(production_capacity: Dict[str, List[Tuple[float, int]]],
                         production_usage: Dict[str, List[Tuple[float, int]]],
                         supply_blocks: List[Tuple[float, bool]]) -> list:

    chart_data = []
    offset = 0
    for structure_type in production_capacity.keys():

        capacity_data = []
        for second, capacity in production_capacity[structure_type]:
            capacity_data.append([int(second * 1000), capacity + offset])

        usage_base_line = [capacity_data[0]]
        for second, usage in production_usage.get(structure_type, []):
            usage_base_line.append([int(second * 1000), offset])

        usage_data = [capacity_data[0]]
        for second, usage in production_usage.get(structure_type, []):
            usage_data.append([int(second * 1000), usage + offset])

        chart_data.append(usage_base_line)
        chart_data.append(usage_data)
        chart_data.append(capacity_data)

        offset += 2 + max(map(lambda x: x[1], production_capacity[structure_type]))

    supply_block_data = []
    was_blocked = False
    for second, blocked in supply_blocks:
        if not was_blocked and blocked:
            supply_block_data.append([int(second * 1000), 0])

        if was_blocked and not blocked:
            supply_block_data.append([int(second * 1000), 0])
            supply_block_data.append([int(second * 1000), "NaN"])

        if blocked:
            supply_block_data.append([int(second * 1000), offset])

        was_blocked = blocked

    chart_data.append(supply_block_data)

    return chart_data


def analyse_replay_file(replay_name: str, replay_file: IO[bytes]) -> dict:
    replay = sc2reader.SC2Reader().load_replay(replay_file)

    data = {"replay_id": replay.filehash, "players": [], "replayName": replay_name}
    for player in replay.players:
        production_capacity = techlabreactor.production_capacity_till_time_for_player(
            600, player, replay)
        production_usage = techlabreactor.production_used_till_time_for_player(600, player, replay)
        supply_blocks = techlabreactor.get_supply_blocks_till_time_for_player(600, player, replay)

        if production_capacity and production_usage:
            data["players"].append({
                "name":
                player.name,
                "structureTypes":
                list(production_capacity.keys()),
                "chartData":
                serialise_chart_data(production_capacity, production_usage, supply_blocks),
            })
    return data



import io
import json
import flask
import terranproduction

try:
    from google.cloud import datastore
    FIREBASE_CONFIG = json.loads(datastore.Client().get(
        datastore.key.Key("Config", "firebaseConfig"))["value"])
except Exception:
    with open("firebaseConfig.json") as local_config:
        FIREBASE_CONFIG = json.load(local_config)

app = flask.Flask(__name__)


@app.route("/")
def index():
    return flask.render_template("index.html.j2")


@app.route("/upload", methods=["POST"])
def upload():
    if flask.request.files:
        file = next(flask.request.files.values(), None)
        replay_name = file.filename
        replay_data_stream = file.stream
    elif flask.request.data:
        replay_name = ""
        replay_data_stream = io.BytesIO(flask.request.data)
    else:
        return flask.abort(400)

    temp_file = terranproduction.util.write_to_temporary_file(replay_data_stream)
    analysis_data = terranproduction.replay.analyse_replay_file(replay_name, temp_file)
    temp_file.close()

    terranproduction.database.post_analysis(FIREBASE_CONFIG, analysis_data)

    return flask.redirect(
        flask.url_for(show_analysis.__name__, replay_id=analysis_data["replay_id"]))


@app.route("/<replay_id>")
def show_analysis(replay_id: str):
    analysis_data = terranproduction.database.get_analysis(FIREBASE_CONFIG, replay_id)
    return flask.render_template("analysis.html.j2", analysis_data=analysis_data)



import json
import pyrebase


def open_db_connection(db_config: dict) -> pyrebase.pyrebase.Database:
    return pyrebase.initialize_app(db_config).database()


def post_analysis(db_config: dict, analysis_data: dict) -> str:
    replay_id = analysis_data["replay_id"]

    db = open_db_connection(db_config)
    db.child("terran_production_analyses").child(replay_id).set(json.dumps(analysis_data))

    return replay_id


def get_analysis(db_config: dict, replay_id: str) -> str:
    db = open_db_connection(db_config)
    analysis_data = db.child("terran_production_analyses").child(replay_id).get().val()
    return analysis_data if analysis_data else {}



import tempfile

CHUNK_SIZE = 1024


def write_to_temporary_file(data_stream):
    replay_file = tempfile.TemporaryFile()
    while True:
        chunk = data_stream.read()
        if not chunk:
            break

        replay_file.write(chunk)

    replay_file.seek(0)
    return replay_file



import setuptools

VERSION = "0.0.2"

setuptools.setup(
    name="terranproduction",
    packages=setuptools.find_packages(),
    package_data={"terranproduction": ["templates/*", "static/styles/*", "static/images/*"]},
    version=VERSION,
    description="SC2 Replay Analyser which visualises a player's Terran Production Efficiency",
    author="Hugo Wainwright",
    author_email="wainwrighthugo@gmail.com",
    url="https://github.com/frugs/allin-terranproduction",
    keywords=["sc2", "replay", "sc2reader"],
    classifiers=[],
    install_requires=[
        "techlabreactor", "requests>=2.19.1", "requests-toolbelt>=0.8.0", "pyrebase", "flask", "google-cloud-datastore"
    ],
)
